@@ -12,36 +12,28 @@
 
 use std::f64::consts::PI;
 
+use crate::QiskitError;
 use hashbrown::HashMap;
+use ndarray::ArrayView2;
 use ndarray::linalg::kron;
 use ndarray::prelude::*;
-use ndarray::ArrayView2;
 use num_complex::Complex64;
-use pyo3::intern;
 use pyo3::prelude::*;
-use pyo3::types::PyList;
 use pyo3::wrap_pyfunction;
-use pyo3::IntoPyObjectExt;
-use pyo3::Python;
-use rand::prelude::*;
-use rand_pcg::Pcg64Mcg;
-use smallvec::SmallVec;
-
 use qiskit_circuit::circuit_data::CircuitData;
 use qiskit_circuit::circuit_instruction::OperationFromPython;
 use qiskit_circuit::converters::dag_to_circuit;
 use qiskit_circuit::dag_circuit::DAGCircuit;
 use qiskit_circuit::gate_matrix::ONE_QUBIT_IDENTITY;
-use qiskit_circuit::imports::QUANTUM_CIRCUIT;
+use qiskit_circuit::instruction::Instruction;
 use qiskit_circuit::operations::StandardGate::{I, X, Y, Z};
-use qiskit_circuit::operations::{Operation, OperationRef, Param, PyInstruction, StandardGate};
-use qiskit_circuit::packed_instruction::{PackedInstruction, PackedOperation};
-use qiskit_circuit::VarsMode;
-
-use crate::QiskitError;
-
+use qiskit_circuit::operations::{Operation, OperationRef, Param, StandardGate};
+use qiskit_circuit::packed_instruction::PackedInstruction;
+use qiskit_circuit::{BlocksMode, VarsMode};
 use qiskit_transpiler::passes::run_optimize_1q_gates_decomposition;
 use qiskit_transpiler::target::Target;
+use rand::prelude::*;
+use rand_pcg::Pcg64Mcg;
 
 static ECR_TWIRL_SET: [([StandardGate; 4], f64); 16] = [
     ([I, Z, Z, Y], 0.),
@@ -226,22 +218,45 @@ fn twirl_gate(
 type CustomGateTwirlingMap = HashMap<String, Vec<([StandardGate; 4], f64)>>;
 
 fn generate_twirled_circuit(
-    py: Python,
     circ: &CircuitData,
     rng: &mut Pcg64Mcg,
     twirling_mask: u8,
     custom_gate_map: Option<&CustomGateTwirlingMap>,
     optimizer_target: Option<&Target>,
 ) -> PyResult<CircuitData> {
-    let mut out_circ = CircuitData::copy_empty_like(circ, VarsMode::Alike)?;
+    let mut out_circ = CircuitData::copy_empty_like(circ, VarsMode::Alike, BlocksMode::Drop)?;
 
-    for inst in circ.data() {
+    for inst in circ.data().iter() {
         if let Some(custom_gate_map) = custom_gate_map {
             if let Some(twirling_set) = custom_gate_map.get(inst.op.name()) {
                 twirl_gate(circ, rng, &mut out_circ, twirling_set.as_slice(), inst)?;
                 continue;
             }
         }
+        if let Some(control_flow) = circ.try_view_control_flow(inst) {
+            let new_blocks: Vec<_> = control_flow
+                .blocks()
+                .into_iter()
+                .map(|block| {
+                    let new_block = generate_twirled_circuit(
+                        block,
+                        rng,
+                        twirling_mask,
+                        custom_gate_map,
+                        optimizer_target,
+                    )?;
+                    Ok(out_circ.add_block(new_block))
+                })
+                .collect::<PyResult<_>>()?;
+            out_circ.push(PackedInstruction::from_control_flow(
+                inst.op.control_flow().clone(),
+                new_blocks,
+                inst.qubits,
+                inst.clbits,
+                inst.label.as_deref().cloned(),
+            ))?;
+            continue;
+        }
         match inst.op.view() {
             OperationRef::StandardGate(gate) => match gate {
                 StandardGate::CX => {
@@ -274,68 +289,6 @@ fn generate_twirled_circuit(
                 }
                 _ => out_circ.push(inst.clone())?,
             },
-            OperationRef::Instruction(py_inst) => {
-                if py_inst.control_flow() {
-                    let new_blocks: PyResult<Vec<PyObject>> = py_inst
-                        .blocks()
-                        .iter()
-                        .map(|block| -> PyResult<PyObject> {
-                            let new_block = generate_twirled_circuit(
-                                py,
-                                block,
-                                rng,
-                                twirling_mask,
-                                custom_gate_map,
-                                optimizer_target,
-                            )?;
-                            new_block.into_py_any(py)
-                        })
-                        .collect();
-                    let new_blocks = new_blocks?;
-                    let blocks_list = PyList::new(
-                        py,
-                        new_blocks.iter().map(|block| {
-                            QUANTUM_CIRCUIT
-                                .get_bound(py)
-                                .call_method1(intern!(py, "_from_circuit_data"), (block,))
-                                .unwrap()
-                        }),
-                    )?;
-
-                    let new_inst_obj = py_inst
-                        .instruction
-                        .bind(py)
-                        .call_method1(intern!(py, "replace_blocks"), (blocks_list,))?
-                        .unbind();
-                    let new_inst = PyInstruction {
-                        qubits: py_inst.qubits,
-                        clbits: py_inst.clbits,
-                        params: py_inst.params,
-                        op_name: py_inst.op_name.clone(),
-                        control_flow: true,
-                        instruction: new_inst_obj.clone_ref(py),
-                    };
-                    let new_inst = PackedInstruction {
-                        op: PackedOperation::from_instruction(Box::new(new_inst)),
-                        qubits: inst.qubits,
-                        clbits: inst.clbits,
-                        params: Some(Box::new(
-                            new_blocks
-                                .iter()
-                                .map(|x| Ok(Param::Obj(x.clone().into_py_any(py)?)))
-                                .collect::<PyResult<SmallVec<[Param; 3]>>>()?,
-                        )),
-                        label: inst.label.clone(),
-                        #[cfg(feature = "cache_pygates")]
-                        py_op: std::sync::OnceLock::new(),
-                    };
-                    #[cfg(feature = "cache_pygates")]
-                    new_inst.py_op.set(new_inst_obj).unwrap();
-                    out_circ.push(new_inst)?;
-                } else {
-                    out_circ.push(inst.clone())?;
-                }
-            }
             _ => {
                 out_circ.push(inst.clone())?;
             }
@@ -353,7 +306,6 @@ fn generate_twirled_circuit(
 #[pyfunction]
 #[pyo3(signature=(circ, twirled_gate=None, custom_twirled_gates=None, seed=None, num_twirls=1, optimizer_target=None))]
 pub(crate) fn twirl_circuit(
-    py: Python,
     circ: &CircuitData,
     twirled_gate: Option<Vec<StandardGate>>,
     custom_twirled_gates: Option<Vec<OperationFromPython>>,
@@ -375,9 +327,10 @@ pub(crate) fn twirl_circuit(
                     StandardGate::ECR => ECR_MASK,
                     StandardGate::ISwap => ISWAP_MASK,
                     _ => {
-                        return Err(QiskitError::new_err(
-                          format!("Provided gate to twirl, {}, is not currently supported you can only use cx, cz, ecr or iswap.", gate.name())
-                        ))
+                        return Err(QiskitError::new_err(format!(
+                            "Provided gate to twirl, {}, is not currently supported you can only use cx, cz, ecr or iswap.",
+                            gate.name()
+                        )));
                     }
                 };
                 out_mask |= new_mask;
@@ -414,7 +367,7 @@ pub(crate) fn twirl_circuit(
                             )
                         )))
                     }
-                    let matrix = gate.operation.matrix(&gate.params);
+                    let matrix = gate.try_matrix();
                     if let Some(matrix) = matrix {
                         let twirl_set = generate_twirling_set(matrix.view());
                         if twirl_set.is_empty() {
@@ -433,7 +386,6 @@ pub(crate) fn twirl_circuit(
     (0..num_twirls)
         .map(|_| {
             generate_twirled_circuit(
-                py,
                 circ,
                 &mut rng,
                 twirling_mask,
