@@ -10,6 +10,7 @@
 // copyright notice, and modified files need to carry a notice indicating
 // that they have been altered from the originals.
 
+use qiskit_circuit::parameter::parameter_expression::ParameterError;
 use qiskit_quantum_info::sparse_observable::ArithmeticError;
 use qiskit_transpiler::target::TargetError;
 use thiserror::Error;
@@ -59,6 +60,12 @@ pub enum ExitCode {
     TargetInvalidInstKey = 304,
     /// Transpilation failed
     TranspilerError = 400,
+    /// QkDag operation error
+    DagError = 500,
+    /// The DAGs have mismatching qubit/clbit amounts during compose.
+    DagComposeMismatch = 501,
+    /// One or more bit indices were not found during compose.
+    DagComposeMissingBit = 502,
 }
 
 impl From<ArithmeticError> for ExitCode {
@@ -96,3 +103,9 @@ impl From<TargetError> for ExitCode {
         }
     }
 }
+
+impl From<ParameterError> for ExitCode {
+    fn from(_value: ParameterError) -> Self {
+        ExitCode::ArithmeticError
+    }
+}
