@@ -13,6 +13,9 @@
 mod pointers;
 
 pub mod circuit;
+pub mod circuit_library;
+pub mod dag;
 pub mod exit_codes;
+pub mod param;
 pub mod sparse_observable;
 pub mod transpiler;
