@@ -14,23 +14,23 @@ use std::hash::Hasher;
 #[cfg(feature = "cache_pygates")]
 use std::sync::OnceLock;
 
-use crate::circuit_instruction::{CircuitInstruction, OperationFromPython};
-use crate::imports::QUANTUM_CIRCUIT;
-use crate::operations::{Operation, OperationRef, Param, PythonOperation};
 use crate::TupleLikeArg;
+use crate::circuit_instruction::{CircuitInstruction, OperationFromPython, extract_params};
+use crate::operations::{Operation, OperationRef, Param, PythonOperation};
 
 use ahash::AHasher;
 use approx::relative_eq;
 use num_complex::Complex64;
 use rustworkx_core::petgraph::stable_graph::NodeIndex;
 
+use crate::instruction::Instruction;
 use numpy::IntoPyArray;
 use numpy::PyArray2;
+use pyo3::IntoPyObjectExt;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::PyTuple;
-use pyo3::IntoPyObjectExt;
-use pyo3::{intern, PyObject, PyResult};
+use pyo3::{PyResult, intern};
 
 /// Parent class for DAGOpNode, DAGInNode, and DAGOutNode.
 #[pyclass(module = "qiskit._accelerate.circuit", subclass)]
@@ -62,7 +62,7 @@ impl DAGNode {
                         Err(_) => {
                             return Err(PyValueError::new_err(
                                 "Invalid node index, must be -1 or a non-negative integer",
-                            ))
+                            ));
                         }
                     };
                     Some(NodeIndex::new(index))
@@ -158,7 +158,7 @@ impl DAGOpNode {
         // like parameter equality are stricter to reject things like
         // Param::Float(0.1) == Param::ParameterExpression(0.1) (if the expression was
         // a python parameter equivalent to a bound value).
-        let Ok(other) = other.downcast::<Self>() else {
+        let Ok(other) = other.cast::<Self>() else {
             return Ok(false);
         };
         let borrowed_other = other.borrow();
@@ -175,35 +175,60 @@ impl DAGOpNode {
         {
             return Ok(false);
         }
-        let params_eq = if slf.instruction.operation.try_standard_gate().is_some() {
-            let mut params_eq = true;
-            for (a, b) in slf
-                .instruction
-                .params
-                .iter()
-                .zip(borrowed_other.instruction.params.iter())
-            {
-                let res = match [a, b] {
-                    [Param::Float(float_a), Param::Float(float_b)] => {
-                        relative_eq!(float_a, float_b, max_relative = 1e-10)
+        let params_eq = match (
+            slf.instruction.operation.view(),
+            borrowed_other.instruction.operation.view(),
+        ) {
+            (OperationRef::StandardGate(_), OperationRef::StandardGate(_))
+            | (OperationRef::StandardInstruction(_), OperationRef::StandardInstruction(_)) => {
+                let slf_params = slf.instruction.params_view();
+                let other_params = borrowed_other.instruction.params_view();
+                let mut params_eq = true;
+                for (a, b) in slf_params.iter().zip(other_params) {
+                    let res = match [a, b] {
+                        [Param::Float(float_a), Param::Float(float_b)] => {
+                            relative_eq!(float_a, float_b, max_relative = 1e-10)
+                        }
+                        [
+                            Param::ParameterExpression(param_a),
+                            Param::ParameterExpression(param_b),
+                        ] => param_a == param_b,
+                        [Param::Obj(param_a), Param::Obj(param_b)] => {
+                            param_a.bind(py).eq(param_b)?
+                        }
+                        _ => false,
+                    };
+                    if !res {
+                        params_eq = false;
+                        break;
                     }
-                    [Param::ParameterExpression(param_a), Param::ParameterExpression(param_b)] => {
-                        param_a == param_b
-                    }
-                    [Param::Obj(param_a), Param::Obj(param_b)] => param_a.bind(py).eq(param_b)?,
-                    _ => false,
-                };
-                if !res {
-                    params_eq = false;
-                    break;
                 }
+                params_eq
+            }
+            (OperationRef::ControlFlow(_), OperationRef::ControlFlow(_)) => {
+                let slf_blocks = slf.instruction.blocks_view();
+                let other_blocks = borrowed_other.instruction.blocks_view();
+                let mut params_eq = true;
+                // TODO: we should be able to do the semantic-equality comparison from Rust
+                // space in the future, without going via Python.  See gh-15267.
+                for (a, b) in slf_blocks.iter().zip(other_blocks) {
+                    if !a
+                        .clone()
+                        .into_py_quantum_circuit(py)?
+                        .eq(b.clone().into_py_quantum_circuit(py)?)?
+                    {
+                        params_eq = false;
+                        break;
+                    }
+                }
+                params_eq
+            }
+            _ => {
+                // For our Python object types (i.e. PyInstruction, PyGate, PyOperation), we've
+                // already evaluated the parameters are equal here via the Python space operation
+                // equality check.
+                true
             }
-            params_eq
-        } else {
-            // We've already evaluated the parameters are equal here via the Python space equality
-            // check so if we're not comparing standard gates and we've reached this point we know
-            // the parameters are already equal.
-            true
         };
 
         Ok(params_eq
@@ -225,15 +250,17 @@ impl DAGOpNode {
         py: Python,
         mut instruction: CircuitInstruction,
         deepcopy: bool,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         if deepcopy {
             instruction.operation = match instruction.operation.view() {
+                OperationRef::ControlFlow(cf) => cf.clone().into(),
                 OperationRef::Gate(gate) => gate.py_deepcopy(py, None)?.into(),
                 OperationRef::Instruction(instruction) => instruction.py_deepcopy(py, None)?.into(),
                 OperationRef::Operation(operation) => operation.py_deepcopy(py, None)?.into(),
                 OperationRef::StandardGate(gate) => gate.into(),
                 OperationRef::StandardInstruction(instruction) => instruction.into(),
                 OperationRef::Unitary(unitary) => unitary.clone().into(),
+                OperationRef::PauliProductMeasurement(ppm) => ppm.clone().into(),
             };
             #[cfg(feature = "cache_pygates")]
             {
@@ -245,7 +272,7 @@ impl DAGOpNode {
         Py::new(py, sub)?.into_py_any(py)
     }
 
-    fn __reduce__(slf: PyRef<Self>, py: Python) -> PyResult<PyObject> {
+    fn __reduce__(slf: PyRef<Self>, py: Python) -> PyResult<Py<PyAny>> {
         let state = slf.as_ref().node.map(|node| node.index());
         let temp = (
             slf.instruction.get_operation(py)?,
@@ -273,6 +300,7 @@ impl DAGOpNode {
         Ok(CircuitInstruction {
             operation: if deepcopy {
                 match self.instruction.operation.view() {
+                    OperationRef::ControlFlow(cf) => cf.clone().into(),
                     OperationRef::Gate(gate) => gate.py_deepcopy(py, None)?.into(),
                     OperationRef::Instruction(instruction) => {
                         instruction.py_deepcopy(py, None)?.into()
@@ -281,6 +309,7 @@ impl DAGOpNode {
                     OperationRef::StandardGate(gate) => gate.into(),
                     OperationRef::StandardInstruction(instruction) => instruction.into(),
                     OperationRef::Unitary(unitary) => unitary.clone().into(),
+                    OperationRef::PauliProductMeasurement(ppm) => ppm.clone().into(),
                 }
             } else {
                 self.instruction.operation.clone()
@@ -295,7 +324,7 @@ impl DAGOpNode {
     }
 
     #[getter]
-    fn get_op(&self, py: Python) -> PyResult<PyObject> {
+    fn get_op(&self, py: Python) -> PyResult<Py<PyAny>> {
         self.instruction.get_operation(py)
     }
 
@@ -349,18 +378,19 @@ impl DAGOpNode {
     }
 
     #[getter]
-    fn get_params(&self) -> &[Param] {
-        self.instruction.params.as_slice()
+    fn get_params(&self, py: Python) -> PyResult<Py<PyAny>> {
+        self.instruction.get_params(py)
     }
 
     #[setter]
-    fn set_params(&mut self, val: smallvec::SmallVec<[crate::operations::Param; 3]>) {
-        self.instruction.params = val;
+    fn set_params(&mut self, val: Bound<PyAny>) -> PyResult<()> {
+        self.instruction.params = extract_params(self.instruction.op(), &val)?;
+        Ok(())
     }
 
     #[getter]
     fn matrix<'py>(&'py self, py: Python<'py>) -> Option<Bound<'py, PyArray2<Complex64>>> {
-        let matrix = self.instruction.operation.matrix(&self.instruction.params);
+        let matrix = self.instruction.try_matrix();
         matrix.map(|mat| mat.into_pyarray(py))
     }
 
@@ -402,20 +432,20 @@ impl DAGOpNode {
 
     #[getter]
     fn definition<'py>(&self, py: Python<'py>) -> PyResult<Option<Bound<'py, PyAny>>> {
-        self.instruction
-            .operation
-            .definition(&self.instruction.params)
-            .map(|data| {
-                QUANTUM_CIRCUIT
-                    .get_bound(py)
-                    .call_method1(intern!(py, "_from_circuit_data"), (data,))
-            })
+        let definition = match self.instruction.operation.view() {
+            OperationRef::StandardGate(g) => g.definition(self.instruction.params_view()),
+            OperationRef::Gate(g) => g.definition(),
+            OperationRef::Instruction(i) => i.definition(),
+            _ => None,
+        };
+        definition
+            .map(|data| data.into_py_quantum_circuit(py))
             .transpose()
     }
 
     /// Sets the Instruction name corresponding to the op for this node
     #[setter]
-    fn set_name(&mut self, py: Python, new_name: PyObject) -> PyResult<()> {
+    fn set_name(&mut self, py: Python, new_name: Py<PyAny>) -> PyResult<()> {
         let op = self.instruction.get_operation_mut(py)?;
         op.setattr(intern!(py, "name"), new_name)?;
         self.instruction.operation = op.extract::<OperationFromPython>()?.operation;
@@ -437,11 +467,11 @@ impl DAGOpNode {
 #[pyclass(module = "qiskit._accelerate.circuit", extends=DAGNode)]
 pub struct DAGInNode {
     #[pyo3(get)]
-    pub wire: PyObject,
+    pub wire: Py<PyAny>,
 }
 
 impl DAGInNode {
-    pub fn new(node: NodeIndex, wire: PyObject) -> (Self, DAGNode) {
+    pub fn new(node: NodeIndex, wire: Py<PyAny>) -> (Self, DAGNode) {
         (DAGInNode { wire }, DAGNode { node: Some(node) })
     }
 }
@@ -449,7 +479,7 @@ impl DAGInNode {
 #[pymethods]
 impl DAGInNode {
     #[new]
-    fn py_new(wire: PyObject) -> PyResult<(Self, DAGNode)> {
+    fn py_new(wire: Py<PyAny>) -> PyResult<(Self, DAGNode)> {
         Ok((DAGInNode { wire }, DAGNode { node: None }))
     }
 
@@ -473,7 +503,7 @@ impl DAGInNode {
     }
 
     fn __eq__(slf: PyRef<Self>, py: Python, other: &Bound<PyAny>) -> PyResult<bool> {
-        match other.downcast::<Self>() {
+        match other.cast::<Self>() {
             Ok(other) => {
                 let borrowed_other = other.borrow();
                 let other_super = borrowed_other.as_ref();
@@ -495,11 +525,11 @@ impl DAGInNode {
 #[pyclass(module = "qiskit._accelerate.circuit", extends=DAGNode)]
 pub struct DAGOutNode {
     #[pyo3(get)]
-    pub wire: PyObject,
+    pub wire: Py<PyAny>,
 }
 
 impl DAGOutNode {
-    pub fn new(node: NodeIndex, wire: PyObject) -> (Self, DAGNode) {
+    pub fn new(node: NodeIndex, wire: Py<PyAny>) -> (Self, DAGNode) {
         (DAGOutNode { wire }, DAGNode { node: Some(node) })
     }
 }
@@ -507,11 +537,11 @@ impl DAGOutNode {
 #[pymethods]
 impl DAGOutNode {
     #[new]
-    fn py_new(wire: PyObject) -> PyResult<(Self, DAGNode)> {
+    fn py_new(wire: Py<PyAny>) -> PyResult<(Self, DAGNode)> {
         Ok((DAGOutNode { wire }, DAGNode { node: None }))
     }
 
-    fn __reduce__(slf: PyRef<Self>, py: Python) -> PyResult<PyObject> {
+    fn __reduce__(slf: PyRef<Self>, py: Python) -> PyResult<Py<PyAny>> {
         let state = slf.as_ref().node.map(|node| node.index());
         (py.get_type::<Self>(), (&slf.wire,), state).into_py_any(py)
     }
@@ -536,7 +566,7 @@ impl DAGOutNode {
     }
 
     fn __eq__(slf: PyRef<Self>, py: Python, other: &Bound<PyAny>) -> PyResult<bool> {
-        match other.downcast::<Self>() {
+        match other.cast::<Self>() {
             Ok(other) => {
                 let borrowed_other = other.borrow();
                 let other_super = borrowed_other.as_ref();
