@@ -14,13 +14,14 @@ use pyo3::prelude::*;
 use pyo3::types::PyAnyMethods;
 use pyo3::{PyResult, Python};
 use qiskit_circuit::circuit_data::{CircuitData, CircuitError};
-use qiskit_circuit::imports;
 use qiskit_circuit::operations::{
-    multiply_param, Operation, OperationRef, Param, PyGate, StandardGate,
+    Operation, OperationRef, Param, PyGate, StandardGate, multiply_param,
 };
+use qiskit_circuit::{BlocksMode, imports};
 use qiskit_circuit::{Clbit, Qubit, VarsMode};
 use smallvec::SmallVec;
 
+use qiskit_circuit::instruction::Parameters;
 use std::f64::consts::PI;
 
 use crate::QiskitError;
@@ -214,7 +215,7 @@ impl CircuitDataForSynthesis for CircuitData {
 
             self.push_packed_operation(
                 inst.op.clone(),
-                inst.params_view(),
+                inst.params.as_deref().cloned(),
                 &remapped_qubits,
                 &remapped_clbits,
             )?;
@@ -228,7 +229,8 @@ impl CircuitDataForSynthesis for CircuitData {
     fn inverse(&self) -> PyResult<CircuitData> {
         let inverse_global_phase = multiply_param(self.global_phase(), -1.0);
 
-        let mut inverse_circuit = CircuitData::copy_empty_like(self, VarsMode::Alike)?;
+        let mut inverse_circuit =
+            CircuitData::copy_empty_like(self, VarsMode::Alike, BlocksMode::Keep)?;
         inverse_circuit.set_global_phase(inverse_global_phase)?;
 
         let data = self.data();
@@ -252,7 +254,7 @@ impl CircuitDataForSynthesis for CircuitData {
 
             inverse_circuit.push_packed_operation(
                 inverse_op.into(),
-                &inverse_op_params,
+                Some(Parameters::Params(inverse_op_params)),
                 self.get_qargs(inst.qubits),
                 self.get_cargs(inst.clbits),
             )?;
@@ -431,7 +433,7 @@ pub fn synth_mcx_noaux_v24(py: Python, num_controls: usize) -> PyResult<CircuitD
 
         circuit.push_packed_operation(
             as_py_gate.into(),
-            &[],
+            None,
             &(0..num_qubits).map(Qubit).collect::<Vec<Qubit>>(),
             &[],
         )?;
