@@ -11,35 +11,38 @@
 // that they have been altered from the originals.
 
 use hashbrown::{HashMap, HashSet};
-use ndarray::linalg::kron;
 use ndarray::Array2;
+use ndarray::linalg::kron;
 use num_complex::Complex64;
 use num_complex::ComplexFloat;
 use qiskit_circuit::object_registry::PyObjectAsKey;
+use qiskit_quantum_info::sparse_observable::PySparseObservable;
+use qiskit_quantum_info::sparse_observable::SparseObservable;
 use smallvec::SmallVec;
 use std::fmt::Debug;
 
 use numpy::PyReadonlyArray2;
+use pyo3::BoundObject;
 use pyo3::exceptions::PyRuntimeError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyBool, PyDict, PyTuple};
-use pyo3::BoundObject;
 
 use qiskit_circuit::circuit_instruction::OperationFromPython;
 use qiskit_circuit::dag_node::DAGOpNode;
 use qiskit_circuit::imports::QI_OPERATOR;
+use qiskit_circuit::instruction::{Instruction, Parameters};
 use qiskit_circuit::object_registry::ObjectRegistry;
 use qiskit_circuit::operations::{
-    Operation, OperationRef, Param, StandardGate, STANDARD_GATE_SIZE,
+    Operation, OperationRef, Param, STANDARD_GATE_SIZE, StandardGate,
 };
 use qiskit_circuit::{Clbit, Qubit};
 use qiskit_quantum_info::unitary_compose;
 use thiserror::Error;
 
+use crate::QiskitError;
 use crate::gate_metrics;
 use crate::standard_gates_commutations;
-use crate::QiskitError;
 
 #[derive(Error, Debug)]
 pub enum CommutationError {
@@ -121,6 +124,113 @@ const fn build_supported_rotations() -> [Option<Option<StandardGate>>; STANDARD_
 static SUPPORTED_ROTATIONS: [Option<Option<StandardGate>>; STANDARD_GATE_SIZE] =
     build_supported_rotations();
 
+/// Try extracting the operator from a Python `PauliGate`. Upon failure, `None` is returned.
+///
+/// # Safety
+///
+/// Behavior is undefined if ``qubits`` contains a qubit index larger equal to ``num_qubits``,
+/// or if the number of Paulis in ``pauli_gate`` does not match the number of indices in ``qubits``.
+fn try_extract_op_from_pauli_gate(
+    pauli_gate: &OperationRef,
+    qubits: &[Qubit],
+    num_qubits: u32,
+) -> Option<SparseObservable> {
+    let OperationRef::Gate(gate) = pauli_gate else {
+        // Gate is called "pauli" but is not a Python gate
+        return None;
+    };
+
+    // Call the internal _extract_sparse_observable method. If the gate doesn't have this,
+    // it means the user passed a Python gate called "pauli" which is not actually a PauliGate.
+    // In that case we return None.
+    Python::attach(|py| {
+        let py_obs = gate
+            .gate
+            .call_method0(py, intern!(py, "_extract_sparse_observable"))
+            .ok()? // Return None if the method didn't exist
+            .cast_bound::<PySparseObservable>(py)
+            .expect("Failed casting to PySparseObservable")
+            .borrow();
+        let local = py_obs.as_inner().expect("Failed to read");
+
+        let out = SparseObservable::identity(num_qubits);
+        Some(out.compose_map(&local, |i| qubits[i as usize].0))
+    })
+}
+
+fn try_extract_op_from_pauli_evo(
+    operation: &OperationRef,
+    qubits: &[Qubit],
+    num_qubits: u32,
+) -> Option<SparseObservable> {
+    let OperationRef::Gate(gate) = operation else {
+        // Gate is called "PauliEvo" but is not a Python gate
+        return None;
+    };
+
+    Python::attach(|py| {
+        let py_obs = gate
+            .gate
+            .call_method0(py, intern!(py, "_extract_sparse_observable"))
+            .ok()? // Return None if the method didn't exist
+            .cast_bound::<PySparseObservable>(py)
+            .expect("Failed casting to PySparseObservable")
+            .borrow();
+        let local = py_obs.as_inner().expect("Failed to read");
+
+        let out = SparseObservable::identity(num_qubits);
+        Some(out.compose_map(&local, |i| qubits[i as usize].0))
+    })
+}
+
+fn try_extract_op_from_ppm(
+    operation: &OperationRef,
+    qubits: &[Qubit],
+    num_qubits: u32,
+) -> Option<SparseObservable> {
+    let OperationRef::PauliProductMeasurement(ppm) = operation else {
+        // Gate is called "pauli_product_measurement" but is not actually a PPM...
+        return None;
+    };
+    let mut indices = Vec::new();
+    let mut bit_terms = Vec::new();
+    for (i, (x_i, z_i)) in ppm.x.iter().zip(ppm.z.iter()).enumerate() {
+        // The only failure case possible here is the identity, because of how we're
+        // constructing the value to convert.
+        let Ok(term) = ::bytemuck::checked::try_cast(((*x_i as u8) << 1) | (*z_i as u8)) else {
+            continue;
+        };
+        indices.push(i as u32);
+        bit_terms.push(term);
+    }
+
+    // The sign of the PPM doesn't matter for commutation checking so we just use 1
+    let coeffs = vec![Complex64::new(1., 0.)];
+    let boundaries = vec![0, bit_terms.len()];
+
+    // SAFETY: We know the data is consistent since we constructed it manually, and that indices are
+    // sorted in ascending order. Hence, calling new_unchecked is safe.
+    let local = unsafe {
+        SparseObservable::new_unchecked(ppm.num_qubits(), coeffs, bit_terms, indices, boundaries)
+    };
+
+    let out = SparseObservable::identity(num_qubits);
+    Some(out.compose_map(&local, |i| qubits[i as usize].0))
+}
+
+fn try_pauli_generator(
+    operation: &OperationRef,
+    qubits: &[Qubit],
+    num_qubits: u32,
+) -> Option<SparseObservable> {
+    match operation.name() {
+        "pauli" => try_extract_op_from_pauli_gate(operation, qubits, num_qubits),
+        "PauliEvolution" => try_extract_op_from_pauli_evo(operation, qubits, num_qubits),
+        "pauli_product_measurement" => try_extract_op_from_ppm(operation, qubits, num_qubits),
+        _ => None,
+    }
+}
+
 fn get_bits_from_py<T>(
     py_bits1: &Bound<'_, PyTuple>,
     py_bits2: &Bound<'_, PyTuple>,
@@ -174,14 +284,15 @@ impl CommutationChecker {
         CommutationChecker::new(Some(library), cache_max_entries, gates)
     }
 
-    #[pyo3(signature=(op1, op2, max_num_qubits=3, approximation_degree=1.))]
+    #[pyo3(signature=(op1, op2, max_num_qubits=None, approximation_degree=1., matrix_max_num_qubits=3))]
     fn commute_nodes(
         &mut self,
         py: Python,
         op1: &DAGOpNode,
         op2: &DAGOpNode,
-        max_num_qubits: u32,
+        max_num_qubits: Option<u32>,
         approximation_degree: f64,
+        matrix_max_num_qubits: u32,
     ) -> PyResult<bool> {
         let (qargs1, qargs2) = get_bits_from_py::<Qubit>(
             op1.instruction.qubits.bind(py),
@@ -194,19 +305,20 @@ impl CommutationChecker {
 
         Ok(self.commute(
             &op1.instruction.operation.view(),
-            &op1.instruction.params,
+            op1.instruction.parameters(),
             &qargs1,
             &cargs1,
             &op2.instruction.operation.view(),
-            &op2.instruction.params,
+            op2.instruction.parameters(),
             &qargs2,
             &cargs2,
             max_num_qubits,
+            matrix_max_num_qubits,
             approximation_degree,
         )?)
     }
 
-    #[pyo3(name="commute", signature=(op1, qargs1, cargs1, op2, qargs2, cargs2, max_num_qubits=3, approximation_degree=1.))]
+    #[pyo3(name="commute", signature=(op1, qargs1, cargs1, op2, qargs2, cargs2, max_num_qubits=None, approximation_degree=1., matrix_max_num_qubits=3))]
     #[allow(clippy::too_many_arguments)]
     fn py_commute(
         &mut self,
@@ -216,22 +328,24 @@ impl CommutationChecker {
         op2: OperationFromPython,
         qargs2: &Bound<'_, PyTuple>,
         cargs2: &Bound<'_, PyTuple>,
-        max_num_qubits: u32,
+        max_num_qubits: Option<u32>,
         approximation_degree: f64,
+        matrix_max_num_qubits: u32,
     ) -> PyResult<bool> {
         let (qargs1, qargs2) = get_bits_from_py::<Qubit>(qargs1, qargs2)?;
         let (cargs1, cargs2) = get_bits_from_py::<Clbit>(cargs1, cargs2)?;
 
         Ok(self.commute(
             &op1.operation.view(),
-            &op1.params,
+            op1.parameters(),
             &qargs1,
             &cargs1,
             &op2.operation.view(),
-            &op2.params,
+            op2.parameters(),
             &qargs2,
             &cargs2,
             max_num_qubits,
+            matrix_max_num_qubits,
             approximation_degree,
         )?)
     }
@@ -260,8 +374,8 @@ impl CommutationChecker {
         Ok(out_dict.unbind())
     }
 
-    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
-        let dict_state = state.downcast_bound::<PyDict>(py)?;
+    fn __setstate__(&mut self, py: Python, state: Py<PyAny>) -> PyResult<()> {
+        let dict_state = state.cast_bound::<PyDict>(py)?;
         self.cache_max_entries = dict_state
             .get_item("cache_max_entries")?
             .unwrap()
@@ -276,7 +390,7 @@ impl CommutationChecker {
         let raw_cache: Bound<PyDict> = dict_state.get_item("cache")?.unwrap().extract()?;
         self.cache = HashMap::with_capacity(raw_cache.len());
         for (key, value) in raw_cache.iter() {
-            let value_dict: &Bound<PyDict> = value.downcast()?;
+            let value_dict: &Bound<PyDict> = value.cast()?;
             self.cache.insert(
                 key.extract()?,
                 commutation_cache_entry_from_pydict(value_dict)?,
@@ -312,17 +426,18 @@ impl CommutationChecker {
     }
 
     #[allow(clippy::too_many_arguments)]
-    pub fn commute(
+    pub fn commute<T>(
         &mut self,
         op1: &OperationRef,
-        params1: &[Param],
+        params1: Option<&Parameters<T>>,
         qargs1: &[Qubit],
         cargs1: &[Clbit],
         op2: &OperationRef,
-        params2: &[Param],
+        params2: Option<&Parameters<T>>,
         qargs2: &[Qubit],
         cargs2: &[Clbit],
-        max_num_qubits: u32,
+        max_num_qubits: Option<u32>,
+        matrix_max_num_qubits: u32,
         approximation_degree: f64,
     ) -> Result<bool, CommutationError> {
         // If the average gate infidelity is below this tolerance, they commute. The tolerance
@@ -330,6 +445,19 @@ impl CommutationChecker {
         // consistency with other places in Qiskit.
         let tol = 1e-12_f64.max(1. - approximation_degree);
 
+        let params1 = params1
+            .map(|p| match p {
+                Parameters::Params(p) => p.as_slice(),
+                _ => &[],
+            })
+            .unwrap_or_default();
+        let params2 = params2
+            .map(|p| match p {
+                Parameters::Params(p) => p.as_slice(),
+                _ => &[],
+            })
+            .unwrap_or_default();
+
         // if we have rotation gates, we attempt to map them to their generators, for example
         // RX -> X or CPhase -> CZ
         let (op1_gate, params1, trivial1) = map_rotation(op1, params1, tol);
@@ -357,21 +485,39 @@ impl CommutationChecker {
             }
         }
 
-        let commutation: Option<bool> = commutation_precheck(
+        let precheck_status = commutation_precheck(
             op1,
-            params1,
             qargs1,
             cargs1,
+            params1,
             op2,
-            params2,
             qargs2,
             cargs2,
+            params2,
             max_num_qubits,
         );
-        if let Some(is_commuting) = commutation {
-            return Ok(is_commuting);
+
+        // Check if we can already return after the pre-check, or if further evaluations are required
+        match precheck_status {
+            PrecheckStatus::Commuting => return Ok(true),
+            PrecheckStatus::NonCommuting => return Ok(false),
+            _ => (),
+        };
+
+        // Handle commutations in between Pauli-based gates, like PauliGate or PauliEvolutionGate
+        let size = qargs1.iter().chain(qargs2.iter()).max().unwrap().0 + 1;
+        if let Some(obs1) = try_pauli_generator(op1, qargs1, size) {
+            if let Some(obs2) = try_pauli_generator(op2, qargs2, size) {
+                return Ok(obs1.commutes(&obs2, tol));
+            }
         }
 
+        // Now there are no more parameterized gates that are allowed
+        if matches!(precheck_status, PrecheckStatus::Parameterized) {
+            return Ok(false);
+        }
+
+        // Sort the arguments.
         let reversed = if op1.num_qubits() != op2.num_qubits() {
             op1.num_qubits() > op2.num_qubits()
         } else {
@@ -406,6 +552,12 @@ impl CommutationChecker {
             is_cachable(first_op, first_params) && is_cachable(second_op, second_params);
 
         if !check_cache {
+            // The arguments are sorted, so if qargs1.len() > matrix_max_num_qubits, then
+            // qargs1.len() > matrix_max_num_qubits as well.
+            if qargs2.len() > matrix_max_num_qubits as usize {
+                return Ok(false);
+            }
+
             return self.commute_matmul(
                 first_op,
                 first_params,
@@ -439,6 +591,12 @@ impl CommutationChecker {
             }
         }
 
+        if qargs1.len() > matrix_max_num_qubits as usize
+            || qargs2.len() > matrix_max_num_qubits as usize
+        {
+            return Ok(false);
+        }
+
         // Perform matrix multiplication to determine commutation
         let is_commuting = self.commute_matmul(
             first_op,
@@ -510,12 +668,12 @@ impl CommutationChecker {
         if first_qarg.len() > second_qarg.len() {
             return Err(CommutationError::FirstInstructionTooLarge);
         };
-        let first_mat = match get_matrix(first_op, first_params) {
+        let first_mat = match try_matrix_with_definition(first_op, first_params, None) {
             Some(matrix) => matrix,
             None => return Ok(false),
         };
 
-        let second_mat = match get_matrix(second_op, second_params) {
+        let second_mat = match try_matrix_with_definition(second_op, second_params, None) {
             Some(matrix) => matrix,
             None => return Ok(false),
         };
@@ -571,35 +729,51 @@ impl CommutationChecker {
     }
 }
 
+/// A pre-check status.
+///
+/// Used to differentiate between the reasons why gates might not commute, which allow
+/// the commutation checker to handle cases individually. E.g. a ``PauliEvolutionGate`` can
+/// still be checked even if the gate is parameterized.
+enum PrecheckStatus {
+    Commuting,     // gates commute for sure
+    NonCommuting,  // gates do not commute for sure
+    Parameterized, // a gate is parameterized and is likely disqualified from being checked
+    Unknown,       // the precheck is inconclusive
+}
+
 #[allow(clippy::too_many_arguments)]
 fn commutation_precheck(
     op1: &OperationRef,
-    params1: &[Param],
     qargs1: &[Qubit],
     cargs1: &[Clbit],
+    params1: &[Param],
     op2: &OperationRef,
-    params2: &[Param],
     qargs2: &[Qubit],
     cargs2: &[Clbit],
-    max_num_qubits: u32,
-) -> Option<bool> {
-    if op1.control_flow() || op2.control_flow() {
-        return Some(false);
+    params2: &[Param],
+    max_num_qubits: Option<u32>,
+) -> PrecheckStatus {
+    if matches!(op1, OperationRef::ControlFlow { .. })
+        || matches!(op2, OperationRef::ControlFlow { .. })
+    {
+        return PrecheckStatus::NonCommuting;
     }
 
     // assuming the number of involved qubits to be small, this might be faster than set operations
     if !qargs1.iter().any(|e| qargs2.contains(e)) && !cargs1.iter().any(|e| cargs2.contains(e)) {
-        return Some(true);
+        return PrecheckStatus::Commuting;
     }
 
-    if qargs1.len() > max_num_qubits as usize || qargs2.len() > max_num_qubits as usize {
-        return Some(false);
+    if let Some(limit) = max_num_qubits {
+        if qargs1.len() > limit as usize || qargs2.len() > limit as usize {
+            return PrecheckStatus::NonCommuting;
+        }
     }
 
     if let OperationRef::StandardGate(gate_1) = op1 {
         if let OperationRef::StandardGate(gate_2) = op2 {
             if SUPPORTED_OP[(*gate_1) as usize] && SUPPORTED_OP[(*gate_2) as usize] {
-                return None;
+                return PrecheckStatus::Unknown;
             }
         }
     }
@@ -611,51 +785,77 @@ fn commutation_precheck(
         op2,
         OperationRef::StandardInstruction(_) | OperationRef::Instruction(_)
     ) {
-        return Some(false);
+        return PrecheckStatus::NonCommuting;
     }
 
     if is_parameterized(params1) || is_parameterized(params2) {
-        return Some(false);
+        return PrecheckStatus::Parameterized;
     }
 
-    None
+    PrecheckStatus::Unknown
 }
 
-fn get_matrix(operation: &OperationRef, params: &[Param]) -> Option<Array2<Complex64>> {
-    if let Some(matrix) = operation.matrix(params) {
-        Some(matrix)
-    } else {
-        match operation {
-            OperationRef::Gate(gate) => Python::with_gil(|py| -> Option<_> {
-                Some(
-                    QI_OPERATOR
-                        .get_bound(py)
-                        .call1((gate.gate.clone_ref(py),))
-                        .ok()?
-                        .getattr(intern!(py, "data"))
-                        .ok()?
-                        .extract::<PyReadonlyArray2<Complex64>>()
-                        .ok()?
-                        .as_array()
-                        .to_owned(),
-                )
-            }),
-            OperationRef::Operation(operation) => Python::with_gil(|py| -> Option<_> {
-                Some(
-                    QI_OPERATOR
-                        .get_bound(py)
-                        .call1((operation.operation.clone_ref(py),))
-                        .ok()?
-                        .getattr(intern!(py, "data"))
-                        .ok()?
-                        .extract::<PyReadonlyArray2<Complex64>>()
-                        .ok()?
-                        .as_array()
-                        .to_owned(),
-                )
-            }),
-            _ => None,
-        }
+/// Returns matrix representation of the specified operation.
+///
+/// For custom python gates:
+/// - The matrix is constructed from the gate's definition when the gate does
+///   not provide a direct `matrix` method.
+/// - To prevent generating excessively large matrices, use
+///   `matrix_from_definition_max_qubits` to set an upper limit on the number
+///   of qubits for which the construction is applied.
+pub fn try_matrix_with_definition(
+    operation: &OperationRef,
+    params: &[Param],
+    matrix_from_definition_max_qubits: Option<u32>,
+) -> Option<Array2<Complex64>> {
+    match operation {
+        OperationRef::StandardGate(gate) => gate.matrix(params),
+        OperationRef::Unitary(unitary) => unitary.matrix(),
+        OperationRef::Gate(gate) => Python::attach(|py| -> Option<_> {
+            if let Some(matrix) = gate.matrix() {
+                return Some(matrix);
+            }
+
+            if matrix_from_definition_max_qubits
+                .is_some_and(|max_qubits| max_qubits < operation.num_qubits())
+            {
+                return None;
+            }
+
+            Some(
+                QI_OPERATOR
+                    .get_bound(py)
+                    .call1((gate.gate.clone_ref(py),))
+                    .ok()?
+                    .getattr(intern!(py, "data"))
+                    .ok()?
+                    .extract::<PyReadonlyArray2<Complex64>>()
+                    .ok()?
+                    .as_array()
+                    .to_owned(),
+            )
+        }),
+        OperationRef::Operation(operation) => Python::attach(|py| -> Option<_> {
+            if matrix_from_definition_max_qubits
+                .is_some_and(|max_qubits| max_qubits < operation.num_qubits())
+            {
+                return None;
+            }
+
+            Some(
+                QI_OPERATOR
+                    .get_bound(py)
+                    .call1((operation.operation.clone_ref(py),))
+                    .ok()?
+                    .getattr(intern!(py, "data"))
+                    .ok()?
+                    .extract::<PyReadonlyArray2<Complex64>>()
+                    .ok()?
+                    .as_array()
+                    .to_owned(),
+            )
+        }),
+        _ => None,
     }
 }
 
@@ -814,14 +1014,16 @@ impl<'py> IntoPyObject<'py> for CommutationLibraryEntry {
     }
 }
 
-impl<'py> FromPyObject<'py> for CommutationLibraryEntry {
-    fn extract_bound(b: &Bound<'py, PyAny>) -> Result<Self, PyErr> {
+impl<'a, 'py> FromPyObject<'a, 'py> for CommutationLibraryEntry {
+    type Error = PyErr;
+
+    fn extract(b: Borrowed<'a, 'py, PyAny>) -> Result<Self, Self::Error> {
         if let Ok(b) = b.extract::<bool>() {
             return Ok(CommutationLibraryEntry::Commutes(b));
         }
-        let dict = b.downcast::<PyDict>()?;
+        let dict = b.cast::<PyDict>()?;
         let mut ret = hashbrown::HashMap::with_capacity(dict.len());
-        for (k, v) in dict {
+        for (k, v) in &*dict {
             let raw_key: SmallVec<[Option<u32>; 2]> = k.extract()?;
             let v: bool = v.extract()?;
             let key = raw_key.into_iter().map(|key| key.map(Qubit)).collect();
@@ -842,8 +1044,8 @@ fn commutation_entry_to_pydict(py: Python, entry: &CommutationCacheEntry) -> PyR
     let out_dict = PyDict::new(py);
     for (k, v) in entry.iter() {
         let qubits = PyTuple::new(py, k.0.iter().map(|q| q.map(|t| t.0)))?;
-        let params0 = PyTuple::new(py, k.1 .0.iter().map(|pk| pk.0))?;
-        let params1 = PyTuple::new(py, k.1 .1.iter().map(|pk| pk.0))?;
+        let params0 = PyTuple::new(py, k.1.0.iter().map(|pk| pk.0))?;
+        let params1 = PyTuple::new(py, k.1.1.iter().map(|pk| pk.0))?;
         out_dict.set_item(
             PyTuple::new(py, [qubits, PyTuple::new(py, [params0, params1])?])?,
             PyBool::new(py, *v),
@@ -857,8 +1059,8 @@ fn commutation_cache_entry_from_pydict(dict: &Bound<PyDict>) -> PyResult<Commuta
     for (k, v) in dict {
         let raw_key: CacheKeyRaw = k.extract()?;
         let qubits = raw_key.0.iter().map(|q| q.map(Qubit)).collect();
-        let params0: SmallVec<_> = raw_key.1 .0;
-        let params1: SmallVec<_> = raw_key.1 .1;
+        let params0: SmallVec<_> = raw_key.1.0;
+        let params1: SmallVec<_> = raw_key.1.1;
         let v: bool = v.extract()?;
         ret.insert((qubits, (params0, params1)), v);
     }
@@ -885,11 +1087,7 @@ impl ParameterKey {
         // because -0 has the sign bit set we'd be hashing 9223372036854775808
         // and be storing it separately from 0. So this normalizes all 0s to
         // be represented by 0
-        if self.0 == 0. {
-            0
-        } else {
-            self.0.to_bits()
-        }
+        if self.0 == 0. { 0 } else { self.0.to_bits() }
     }
 }
 
