@@ -10,28 +10,27 @@
 // copyright notice, and modified files need to carry a notice indicating
 // that they have been altered from the originals.
 
-use std::sync::OnceLock;
-
+use super::errors::BasisTranslatorError;
 use hashbrown::HashMap;
 use indexmap::{IndexMap, IndexSet};
 use pyo3::prelude::*;
+use qiskit_circuit::Qubit;
 use qiskit_circuit::bit::QuantumRegister;
+use qiskit_circuit::circuit_data::CircuitData;
 use qiskit_circuit::circuit_instruction::OperationFromPython;
 use qiskit_circuit::imports::GATE;
-use qiskit_circuit::operations::{get_standard_gate_names, StandardGate, StandardInstruction};
+use qiskit_circuit::instruction::{Instruction, Parameters};
+use qiskit_circuit::operations::{StandardGate, StandardInstruction, get_standard_gate_names};
 use qiskit_circuit::packed_instruction::PackedOperation;
 use qiskit_circuit::parameter::parameter_expression::ParameterExpression;
 use qiskit_circuit::parameter::symbol_expr::Symbol;
 use qiskit_circuit::parameter_table::ParameterUuid;
-use qiskit_circuit::Qubit;
 use qiskit_circuit::{
-    circuit_data::CircuitData,
     dag_circuit::DAGCircuit,
     operations::{Operation, Param},
 };
 use smallvec::SmallVec;
-
-use super::errors::BasisTranslatorError;
+use std::sync::OnceLock;
 
 // Custom types
 pub type GateIdentifier = (String, u32);
@@ -74,13 +73,13 @@ pub(super) fn compose_transforms<'a>(
         let gate = if let Some(op) = name_to_packed_operation(&gate_name, gate_num_qubits) {
             op
         } else {
-            let extract_py = Python::with_gil(|py| -> PyResult<OperationFromPython> {
+            let extract_py = Python::attach(|py| -> PyResult<OperationFromPython> {
                 GATE.get_bound(py)
                     .call1((&gate_name, gate_num_qubits, placeholder_params.as_ref()))?
                     .extract()
             })
             .unwrap_or_else(|_| panic!("Error creating custom gate for entry {}", gate_name));
-            placeholder_params = extract_py.params;
+            placeholder_params = extract_py.params_view().iter().cloned().collect();
             extract_py.operation
         };
         let qubits: Vec<Qubit> = (0..dag.num_qubits() as u32).map(Qubit).collect();
@@ -88,11 +87,7 @@ pub(super) fn compose_transforms<'a>(
             gate,
             &qubits,
             &[],
-            if placeholder_params.is_empty() {
-                None
-            } else {
-                Some(placeholder_params.clone())
-            },
+            Some(Parameters::Params(placeholder_params.clone())),
             None,
             #[cfg(feature = "cache_pygates")]
             None,
@@ -150,7 +145,7 @@ pub(super) fn compose_transforms<'a>(
                                 "Error converting circuit to dag".to_string(),
                             )
                         })?;
-                dag.substitute_node_with_dag(node, &replace_dag, None, None, None)
+                dag.substitute_node_with_dag(node, &replace_dag, None, None, None, None)
                     .map_err(|_| {
                         BasisTranslatorError::BasisDAGCircuitError(
                             "Error during node substitution with DAG.".to_string(),
@@ -198,37 +193,19 @@ fn get_gates_num_params(
     example_gates: &mut IndexMap<GateIdentifier, usize, ahash::RandomState>,
 ) {
     for (_, inst) in dag.op_nodes(true) {
-        example_gates.insert(
-            (inst.op.name().to_string(), inst.op.num_qubits()),
-            inst.params_view().len(),
-        );
-        if inst.op.control_flow() {
-            let blocks = inst.op.blocks();
-            for block in blocks {
-                get_gates_num_params_circuit(&block, example_gates);
-            }
-        }
-    }
-}
-
-/// `CircuitData` variant.
-///
-/// Gets the identifier of a gate instance (name, number of qubits) mapped to the
-/// number of parameters it contains currently.
-fn get_gates_num_params_circuit(
-    circuit: &CircuitData,
-    example_gates: &mut IndexMap<GateIdentifier, usize, ahash::RandomState>,
-) {
-    for inst in circuit.iter() {
-        example_gates.insert(
-            (inst.op.name().to_string(), inst.op.num_qubits()),
-            inst.params_view().len(),
-        );
-        if inst.op.control_flow() {
-            let blocks = inst.op.blocks();
-            for block in blocks {
-                get_gates_num_params_circuit(&block, example_gates);
+        if let Some(control_flow) = dag.try_view_control_flow(inst) {
+            example_gates.insert(
+                (inst.op.name().to_string(), inst.op.num_qubits()),
+                inst.op.num_params() as usize,
+            );
+            for block in control_flow.blocks() {
+                get_gates_num_params(block, example_gates);
             }
+        } else {
+            example_gates.insert(
+                (inst.op.name().to_string(), inst.op.num_qubits()),
+                inst.params_view().len(),
+            );
         }
     }
 }
