@@ -10,20 +10,16 @@
 // copyright notice, and modified files need to carry a notice indicating
 // that they have been altered from the originals.
 
-use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::wrap_pyfunction;
 
-use qiskit_circuit::circuit_data::CircuitData;
 use qiskit_circuit::dag_circuit::DAGCircuit;
-use qiskit_circuit::imports::CIRCUIT_TO_DAG;
-use qiskit_circuit::operations::{Operation, OperationRef};
+use qiskit_circuit::operations::Operation;
 use qiskit_circuit::{PhysicalQubit, Qubit};
 
 use crate::target::Target;
 
 fn recurse(
-    py: Python,
     dag: &DAGCircuit,
     target: &Target,
     wire_map: Option<&[Qubit]>,
@@ -46,33 +42,23 @@ fn recurse(
         }
     };
 
-    for (_node, inst) in dag.op_nodes(false) {
+    for (_, inst) in dag.op_nodes(false) {
         let qubits = dag.get_qargs(inst.qubits);
-        if inst.op.control_flow() {
-            if let OperationRef::Instruction(py_inst) = inst.op.view() {
-                let raw_blocks = py_inst.instruction.getattr(py, "blocks")?;
-                let circuit_to_dag = CIRCUIT_TO_DAG.get_bound(py);
-                for raw_block in raw_blocks.bind(py).try_iter()? {
-                    let block_obj = raw_block?;
-                    let block = block_obj
-                        .getattr(intern!(py, "_data"))?
-                        .downcast::<CircuitData>()?
-                        .borrow();
-                    let new_dag: DAGCircuit =
-                        circuit_to_dag.call1((block_obj.clone(),))?.extract()?;
-                    let wire_map = (0..block.num_qubits())
-                        .map(|inner| {
-                            let outer = qubits[inner];
-                            match wire_map {
-                                Some(wire_map) => wire_map[outer.index()],
-                                None => outer,
-                            }
-                        })
-                        .collect::<Vec<_>>();
-                    let res = recurse(py, &new_dag, target, Some(&wire_map))?;
-                    if res.is_some() {
-                        return Ok(res);
-                    }
+        if let Some(control_flow) = dag.try_view_control_flow(inst) {
+            for block in control_flow.blocks() {
+                let wire_map = (0..block.num_qubits())
+                    .map(|inner| {
+                        let outer = qubits[inner];
+                        match wire_map {
+                            Some(wire_map) => wire_map[outer.index()],
+                            None => outer,
+                        }
+                    })
+                    .collect::<Vec<_>>();
+
+                let res = recurse(block, target, Some(&wire_map))?;
+                if res.is_some() {
+                    return Ok(res);
                 }
             }
         } else if qubits.len() == 2 && !check_qubits(qubits) {
@@ -87,13 +73,9 @@ fn recurse(
 
 #[pyfunction]
 #[pyo3(name = "check_map")]
-pub fn py_run_check_map(
-    py: Python,
-    dag: &DAGCircuit,
-    target: &Target,
-) -> PyResult<Option<(String, [u32; 2])>> {
+pub fn py_run_check_map(dag: &DAGCircuit, target: &Target) -> PyResult<Option<(String, [u32; 2])>> {
     if dag.has_control_flow() {
-        recurse(py, dag, target, None)
+        recurse(dag, target, None)
     } else {
         Ok(run_check_map(dag, target)
             .map(|(name, qubits)| (name.to_string(), [qubits[0].0, qubits[1].0])))
