@@ -11,11 +11,9 @@
 // that they have been altered from the originals.
 
 mod dag;
-mod distance;
 pub mod heuristic;
 mod layer;
 mod layout;
-mod neighbors;
 pub(crate) mod route;
 
 use pyo3::prelude::*;
