@@ -273,6 +273,14 @@ Operations are non-reversible changes in the quantum state of the circuit.
 * :class:`~qiskit.circuit.Measure`
 * :class:`~qiskit.circuit.Reset`
 
+In addition to :class:`.Measure`, there is a generalized joint-Pauli measurement available, used as
+a basic operation in Pauli-based computation.
+
+.. autosummary::
+    :toctree: ../stubs
+
+    PauliProductMeasurement
+
 
 .. _generalized-gates:
 
@@ -898,6 +906,7 @@ from .generalized_gates import (
     UCRZGate,
 )
 from .pauli_evolution import PauliEvolutionGate
+from .pauli_product_measurement import PauliProductMeasurement
 from .hamiltonian_gate import HamiltonianGate
 from .boolean_logic import (
     AND,
