@@ -29,7 +29,7 @@ class ZFeatureMap(PauliFeatureMap):
         ┌───┐┌─────────────┐┌───┐┌─────────────┐
         ┤ H ├┤ P(2.0*x[0]) ├┤ H ├┤ P(2.0*x[0]) ├
         ├───┤├─────────────┤├───┤├─────────────┤
-        ┤ H ├┤ U(2.0*x[1]) ├┤ H ├┤ P(2.0*x[1]) ├
+        ┤ H ├┤ P(2.0*x[1]) ├┤ H ├┤ P(2.0*x[1]) ├
         ├───┤├─────────────┤├───┤├─────────────┤
         ┤ H ├┤ P(2.0*x[2]) ├┤ H ├┤ P(2.0*x[2]) ├
         └───┘└─────────────┘└───┘└─────────────┘
