@@ -15,20 +15,23 @@
 from qiskit.circuit.quantumcircuit import QuantumCircuit
 
 
-def clifford_2_3():
+def clifford_4_2():
     """
-    Clifford template 2_3:
+    Clifford template 4_2:
 
     .. code-block:: text
 
-             ┌───┐┌───┐
-        q_0: ┤ H ├┤ H ├
-             └───┘└───┘
+        q_0: ───────■────────■─
+             ┌───┐┌─┴─┐┌───┐ │
+        q_1: ┤ H ├┤ X ├┤ H ├─■─
+             └───┘└───┘└───┘
 
     Returns:
         QuantumCircuit: template as a quantum circuit.
     """
-    qc = QuantumCircuit(1)
-    qc.h(0)
-    qc.h(0)
+    qc = QuantumCircuit(2)
+    qc.h(1)
+    qc.cx(0, 1)
+    qc.h(1)
+    qc.cz(0, 1)
     return qc
