@@ -497,7 +497,7 @@ used to transform any :class:`.PhaseGate` outside the specified angle bounds. Yo
 need to write a function that takes in the angle values for the gate and returns
 a :class:`.DAGCircuit`. For example::
 
-    from qiskit.transpiler.passes.utils.wrap_angles import WRAP_ANGLE_REGISTRY
+    from qiskit.transpiler.passes import WrapAngles
 
     def fold_phase(angles: List[float], qubits: List[int]) -> DAGCircuit:
         angle = angles[0]
@@ -511,7 +511,7 @@ a :class:`.DAGCircuit`. For example::
             dag.apply_operation_back(PhaseGate(math.pi), [dag.qubits[0]])
         return dag
 
-    WRAP_ANGLE_REGISTRY.add_wrapper("phase", fold_phase)
+    WrapAngles.DEFAULT_REGISTRY.add_wrapper("phase", fold_phase)
 
 This function will transform the out of bounds gates into one that respects the angle
 bounds in the target and the target's other constraints (although not particularly well).
@@ -526,11 +526,9 @@ is used to actually submit circuits to a device or simulator. The run method
 handles submitting the circuits to the backend to be executed and returning a
 :class:`~qiskit.providers.Job` object. Depending on the type of backend this
 typically involves serializing the circuit object into the API format used by a
-backend. For example, on IBM backends from the ``qiskit-ibm-provider``
-package this involves converting from a quantum circuit and options into a
-:mod:`.qpy` payload embedded in JSON and submitting that to the IBM Quantum
-API. Since every backend interface is different (and in the case of the local
-simulators serialization may not be needed) it is expected that the backend's
+backend.
+Since backend serialization needs might differ (and, in the case of local
+simulators, serialization may not even be needed), it is expected that the backend’s
 :obj:`~qiskit.providers.BackendV2.run` method will handle this conversion.
 
 An example run method would be something like::
