@@ -18,6 +18,7 @@ import collections.abc
 import io
 import struct
 import uuid
+from dataclasses import dataclass
 
 import numpy as np
 
@@ -26,15 +27,28 @@ from qiskit.circuit.classical import expr, types
 from qiskit.circuit.parameter import Parameter
 from qiskit.circuit.parameterexpression import (
     ParameterExpression,
+    ParameterValueType,
+    OpCode,
     op_code_to_method,
-    _OPCode,
-    _SUBS,
 )
 from qiskit.circuit.parametervector import ParameterVector, ParameterVectorElement
 from qiskit.qpy import common, formats, exceptions, type_keys
 from qiskit.qpy.binary_io.parse_sympy_repr import parse_sympy_repr
 
 
+@dataclass
+class _INSTRUCTION:
+    op: OpCode
+    lhs: ParameterValueType | None
+    rhs: ParameterValueType | None = None
+
+
+@dataclass
+class _SUBS:
+    binds: dict
+    op: OpCode = OpCode.SUBSTITUTE
+
+
 def _write_parameter(file_obj, obj):
     name_bytes = obj.name.encode(common.ENCODE)
     file_obj.write(struct.pack(formats.PARAMETER_PACK, len(name_bytes), obj.uuid.bytes))
@@ -42,14 +56,14 @@ def _write_parameter(file_obj, obj):
 
 
 def _write_parameter_vec(file_obj, obj):
-    name_bytes = obj._vector._name.encode(common.ENCODE)
+    name_bytes = obj.vector._name.encode(common.ENCODE)
     file_obj.write(
         struct.pack(
             formats.PARAMETER_VECTOR_ELEMENT_PACK,
             len(name_bytes),
-            len(obj._vector),
+            len(obj.vector),
             obj.uuid.bytes,
-            obj._index,
+            obj.index,
         )
     )
     file_obj.write(name_bytes)
@@ -155,7 +169,7 @@ def _write_parameter_expression_v13(file_obj, obj, version):
         rhs_type, rhs = _encode_replay_entry(inst.rhs, file_obj, version, True)
         entry = struct.pack(
             formats.PARAM_EXPR_ELEM_V13_PACK,
-            inst.op,
+            int(inst.op),
             lhs_type.encode("utf8"),
             lhs,
             rhs_type.encode("utf8"),
@@ -170,7 +184,9 @@ def _write_parameter_expression(file_obj, obj, use_symengine, *, version):
     with io.BytesIO() as buf:
         extra_symbols = _write_parameter_expression_v13(buf, obj, version)
         expr_bytes = buf.getvalue()
-    symbol_table_len = len(obj._parameter_symbols)
+
+    parameters = obj.parameters
+    symbol_table_len = len(parameters)
     if extra_symbols:
         symbol_table_len += 2 * len(extra_symbols)
     param_expr_header_raw = struct.pack(
@@ -178,7 +194,7 @@ def _write_parameter_expression(file_obj, obj, use_symengine, *, version):
     )
     file_obj.write(param_expr_header_raw)
     file_obj.write(expr_bytes)
-    for symbol, value in obj._parameter_symbols.items():
+    for symbol in parameters:
         symbol_key = type_keys.Value.assign(symbol)
 
         # serialize key
@@ -188,11 +204,8 @@ def _write_parameter_expression(file_obj, obj, use_symengine, *, version):
             symbol_data = common.data_to_binary(symbol, _write_parameter)
 
         # serialize value
-        if value == symbol._symbol_expr:
-            value_key = symbol_key
-            value_data = bytes()
-        else:
-            value_key, value_data = dumps_value(value, version=version, use_symengine=use_symengine)
+        value_key = symbol_key
+        value_data = bytes()
 
         elem_header = struct.pack(
             formats.PARAM_EXPR_MAP_ELEM_V3_PACK,
@@ -225,35 +238,6 @@ def _write_parameter_expression(file_obj, obj, use_symengine, *, version):
             file_obj.write(elem_header)
             file_obj.write(symbol_data)
             file_obj.write(value_data)
-        for symbol in extra_symbols.values():
-            symbol_key = type_keys.Value.assign(symbol)
-            # serialize key
-            if symbol_key == type_keys.Value.PARAMETER_VECTOR:
-                symbol_data = common.data_to_binary(symbol, _write_parameter_vec)
-            elif symbol_key == type_keys.Value.PARAMETER_EXPRESSION:
-                symbol_data = common.data_to_binary(
-                    symbol,
-                    _write_parameter_expression,
-                    use_symengine=use_symengine,
-                    version=version,
-                )
-            else:
-                symbol_data = common.data_to_binary(symbol, _write_parameter)
-            # serialize value
-
-            value_key, value_data = dumps_value(
-                symbol, version=version, use_symengine=use_symengine
-            )
-
-            elem_header = struct.pack(
-                formats.PARAM_EXPR_MAP_ELEM_V3_PACK,
-                symbol_key,
-                value_key,
-                len(value_data),
-            )
-            file_obj.write(elem_header)
-            file_obj.write(symbol_data)
-            file_obj.write(value_data)
 
 
 class _ExprWriter(expr.ExprVisitor[None]):
@@ -340,6 +324,10 @@ class _ExprWriter(expr.ExprVisitor[None]):
                 struct.pack(formats.EXPR_VALUE_FLOAT_PACK, *formats.EXPR_VALUE_FLOAT(node.value))
             )
         elif isinstance(node.value, Duration):
+            if self.version < 16 and node.value.unit() == "ps":
+                raise exceptions.UnsupportedFeatureForVersion(
+                    "Duration variant 'Duration.ps'", required=16, target=self.version
+                )
             self.file_obj.write(type_keys.ExprValue.DURATION)
             _write_duration(self.file_obj, node.value)
         else:
@@ -422,6 +410,11 @@ def _write_duration(file_obj, duration: Duration):
         file_obj.write(
             struct.pack(formats.DURATION_DT_PACK, *formats.DURATION_DT(duration.value()))
         )
+    elif unit == "ps":
+        file_obj.write(type_keys.CircuitDuration.PS)
+        file_obj.write(
+            struct.pack(formats.DURATION_PS_PACK, *formats.DURATION_PS(duration.value()))
+        )
     elif unit == "ns":
         file_obj.write(type_keys.CircuitDuration.NS)
         file_obj.write(
@@ -485,7 +478,7 @@ def _read_parameter_expression(file_obj):
 
     sympy_str = file_obj.read(data.expr_size).decode(common.ENCODE)
     expr_ = parse_sympy_repr(sympy_str)
-    symbol_map = {}
+    name_map = {}
     for _ in range(data.map_elements):
         elem_data = formats.PARAM_EXPR_MAP_ELEM(
             *struct.unpack(
@@ -504,14 +497,14 @@ def _read_parameter_expression(file_obj):
         elif elem_key == type_keys.Value.COMPLEX:
             value = complex(*struct.unpack(formats.COMPLEX_PACK, binary_data))
         elif elem_key == type_keys.Value.PARAMETER:
-            value = symbol._symbol_expr
+            value = symbol
         elif elem_key == type_keys.Value.PARAMETER_EXPRESSION:
             value = common.data_from_binary(binary_data, _read_parameter_expression)
         else:
             raise exceptions.QpyError(f"Invalid parameter expression map type: {elem_key}")
-        symbol_map[symbol] = value
+        name_map[symbol.name] = value
 
-    return ParameterExpression(symbol_map, str(expr_))
+    return ParameterExpression(name_map, str(expr_))
 
 
 def _read_parameter_expression_v3(file_obj, vectors, use_symengine):
@@ -526,7 +519,7 @@ def _read_parameter_expression_v3(file_obj, vectors, use_symengine):
         sympy_str = payload.decode(common.ENCODE)
         expr_ = parse_sympy_repr(sympy_str)
 
-    symbol_map = {}
+    name_map = {}
     for _ in range(data.map_elements):
         elem_data = formats.PARAM_EXPR_MAP_ELEM_V3(
             *struct.unpack(
@@ -552,7 +545,7 @@ def _read_parameter_expression_v3(file_obj, vectors, use_symengine):
         elif elem_key == type_keys.Value.COMPLEX:
             value = complex(*struct.unpack(formats.COMPLEX_PACK, binary_data))
         elif elem_key in (type_keys.Value.PARAMETER, type_keys.Value.PARAMETER_VECTOR):
-            value = symbol._symbol_expr
+            value = symbol
         elif elem_key == type_keys.Value.PARAMETER_EXPRESSION:
             value = common.data_from_binary(
                 binary_data,
@@ -562,9 +555,9 @@ def _read_parameter_expression_v3(file_obj, vectors, use_symengine):
             )
         else:
             raise exceptions.QpyError(f"Invalid parameter expression map type: {elem_key}")
-        symbol_map[symbol] = value
+        name_map[symbol.name] = value
 
-    return ParameterExpression(symbol_map, str(expr_))
+    return ParameterExpression(name_map, str(expr_))
 
 
 def _read_parameter_expression_v13(file_obj, vectors, version):
@@ -603,7 +596,7 @@ def _read_parameter_expression_v13(file_obj, vectors, version):
         elif elem_key == type_keys.Value.COMPLEX:
             value = complex(*struct.unpack(formats.COMPLEX_PACK, binary_data))
         elif elem_key in (type_keys.Value.PARAMETER, type_keys.Value.PARAMETER_VECTOR):
-            value = symbol._symbol_expr
+            value = symbol
         elif elem_key == type_keys.Value.PARAMETER_EXPRESSION:
             value = common.data_from_binary(
                 binary_data,
@@ -686,13 +679,13 @@ def _read_parameter_expr_v13(buf, symbol_map, version, vectors):
             )
         if expression_data.OP_CODE == 255:
             continue
-        method_str = op_code_to_method(_OPCode(expression_data.OP_CODE))
+        method_str = op_code_to_method(expression_data.OP_CODE)
         if expression_data.OP_CODE in {0, 1, 2, 3, 4, 13, 15, 18, 19, 20}:
             rhs = stack.pop()
             lhs = stack.pop()
             # Reverse ops for commutative ops, which are add, mul (0 and 2 respectively)
             # op codes 13 and 15 can never be reversed and 18, 19, 20
-            # are the reversed versions of non-commuative operations
+            # are the reversed versions of non-commutative operations
             # so 1, 3, 4 and 18, 19, 20 handle this explicitly.
             if (
                 not isinstance(lhs, ParameterExpression)
@@ -710,6 +703,7 @@ def _read_parameter_expr_v13(buf, symbol_map, version, vectors):
             lhs = stack.pop()
             stack.append(getattr(lhs, method_str)())
         data = buf.read(formats.PARAM_EXPR_ELEM_V13_SIZE)
+
     return stack.pop()
 
 
@@ -843,6 +837,11 @@ def _read_duration(file_obj) -> Duration:
             struct.unpack(formats.DURATION_DT_PACK, file_obj.read(formats.DURATION_DT_SIZE))
         )
         return Duration.dt(elem.value)
+    if type_key == type_keys.CircuitDuration.PS:
+        elem = formats.DURATION_PS._make(
+            struct.unpack(formats.DURATION_PS_PACK, file_obj.read(formats.DURATION_PS_SIZE))
+        )
+        return Duration.ps(elem.value)
     if type_key == type_keys.CircuitDuration.NS:
         elem = formats.DURATION_NS._make(
             struct.unpack(formats.DURATION_NS_PACK, file_obj.read(formats.DURATION_NS_SIZE))
