@@ -41,6 +41,65 @@ logger = logging.getLogger(__name__)
 class TestStatevector(QiskitTestCase):
     """Tests for Statevector class."""
 
+    def test_from_circuit_bell_state(self):
+        """Test from_circuit creates Bell state correctly."""
+        qc = QuantumCircuit(2)
+        qc.h(0)
+        qc.cx(0, 1)
+
+        sv = Statevector.from_circuit(qc)
+        expected_data = np.array([1 / np.sqrt(2), 0, 0, 1 / np.sqrt(2)])
+        expected = Statevector(expected_data)
+
+        self.assertTrue(sv.equiv(expected))
+
+    def test_from_circuit_transpilation_consistency(self):
+        """Statevector.from_circuit gives consistent results under transpilation."""
+
+        qc = QuantumCircuit(3)
+        qc.h(0)
+        qc.cx(0, 1)
+        qc.swap(1, 2)
+        qc.z(0)
+
+        backend = BasicSimulator()
+        transpiled_opt0 = transpile(qc, backend, optimization_level=0)
+        transpiled_opt2 = transpile(qc, backend, optimization_level=2)
+
+        sv1 = Statevector.from_circuit(transpiled_opt0)
+        sv2 = Statevector.from_circuit(transpiled_opt2)
+
+        self.assertTrue(sv1.equiv(sv2))
+
+        sv2_nolayout = Statevector.from_circuit(transpiled_opt2, ignore_set_layout=True)
+        self.assertFalse(sv1.equiv(sv2_nolayout))
+
+    def test_from_circuit_vs_manual_method(self):
+        """Test from_circuit matches manual operator evolution."""
+        qc = QuantumCircuit(2)
+        qc.h(0)
+        qc.cx(0, 1)
+
+        sv1 = Statevector.from_circuit(qc)
+
+        initial_state = Statevector.from_label("00")
+        op = Operator.from_circuit(qc)
+        sv2 = initial_state.evolve(op)
+
+        self.assertTrue(sv1.equiv(sv2))
+
+    def test_from_circuit_ignore_layout(self):
+        """Test from_circuit with ignore_set_layout parameter."""
+        qc = QuantumCircuit(2)
+        qc.h(0)
+        qc.cx(0, 1)
+
+        # Without layout, both should give same result
+        sv1 = Statevector.from_circuit(qc, ignore_set_layout=False)
+        sv2 = Statevector.from_circuit(qc, ignore_set_layout=True)
+
+        self.assertTrue(sv1.equiv(sv2))
+
     @classmethod
     def rand_vec(cls, n, normalize=False):
         """Return complex vector or statevector"""
@@ -150,7 +209,7 @@ class TestStatevector(QiskitTestCase):
         qc.x(0)
         qc.h(1)
         gate = qc.to_gate()
-        gate_ctrl = gate.control()
+        gate_ctrl = gate.control(annotated=False)
 
         circuit = QuantumCircuit(3)
         circuit.x(0)
@@ -277,6 +336,16 @@ class TestStatevector(QiskitTestCase):
             evolved = Statevector(vec).evolve(op)
             self.assertEqual(target, evolved)
 
+    def test_evolve_operator_overload_dimensions(self):
+        """Test that the @ operator returns a Statevector of correct dimension, type and value."""
+        op = random_unitary(4)  # 4x4 unitary
+        vec = Statevector(self.rand_vec(4))  # 4-dim state vector
+        result = op @ vec
+        target = op.data @ vec.data
+        self.assertIsInstance(result, Statevector)
+        self.assertEqual(result.data.shape, (4,))
+        self.assertTrue(np.array_equal(result.data, target))
+
     def test_evolve_subsystem(self):
         """Test subsystem _evolve method."""
         # Test evolving single-qubit of 3-qubit system
@@ -486,6 +555,14 @@ class TestStatevector(QiskitTestCase):
                     target[key] = 2 * i + j + 1
             self.assertDictAlmostEqual(target, vec.to_dict())
 
+    def test_to_dict_decimals(self):
+        """Test to_dict method with decimals argument."""
+        decimal = 3
+        sv = np.array([1 / np.sqrt(2), 0, 0, -1 / np.sqrt(2)], dtype=np.complex128)
+        vec_rounded = Statevector(sv).to_dict(decimals=decimal)
+        expected = {"00": 0.707, "11": -0.707}
+        self.assertEqual(vec_rounded, expected)
+
     def test_probabilities_product(self):
         """Test probabilities method for product state"""
 
@@ -1167,6 +1244,18 @@ class TestStatevector(QiskitTestCase):
         self.assertAlmostEqual(expval_state_1_n1, 4 + 0j)
         self.assertAlmostEqual(expval_state_1_n2, 5 + 0j)
 
+        for prefix, factor in [("", 1), ("-", -1), ("i", 1j), ("-i", -1j)]:
+            pauli_label = f"{prefix}I"
+            identity_pauli_1 = Pauli(pauli_label)
+            with self.subTest(pauli=pauli_label):
+                self.assertAlmostEqual(state_1.expectation_value(identity_pauli_1), factor + 0j)
+                self.assertAlmostEqual(
+                    state_1_n1.expectation_value(identity_pauli_1), factor * 4 + 0j
+                )
+                self.assertAlmostEqual(
+                    state_1_n2.expectation_value(identity_pauli_1), factor * 5 + 0j
+                )
+
         # Let's try a multi-qubit case
         n_qubits = 3
         state_coeff = 3 - 4j
@@ -1177,6 +1266,20 @@ class TestStatevector(QiskitTestCase):
         target = op_coeff * np.abs(state_coeff) ** 2
         self.assertAlmostEqual(expval, target)
 
+        coeffs = np.array([1 + 2j, -3j, 4])
+        op_multi = SparsePauliOp.from_list([("I" * n_qubits, coeff) for coeff in coeffs])
+        expval_multi = state_test.expectation_value(op_multi)
+        target_multi = np.sum(coeffs) * np.abs(state_coeff) ** 2
+        self.assertAlmostEqual(expval_multi, target_multi)
+
+        for prefix, factor in [("", 1), ("-", -1), ("i", 1j), ("-i", -1j)]:
+            pauli_label = f"{prefix}{'I' * n_qubits}"
+            identity_pauli_n = Pauli(pauli_label)
+            with self.subTest(pauli=pauli_label):
+                expval_pauli = state_test.expectation_value(identity_pauli_n)
+                target_pauli = factor * np.abs(state_coeff) ** 2
+                self.assertAlmostEqual(expval_pauli, target_pauli)
+
     @data(*(qargs for i in range(4) for qargs in permutations(range(4), r=i + 1)))
     def test_probabilities_qargs(self, qargs):
         """Test probabilities method with qargs"""
