@@ -23,6 +23,7 @@ import sys
 
 import numpy as np
 
+import qiskit
 from qiskit import QuantumCircuit, QuantumRegister, ClassicalRegister
 from qiskit.circuit import Clbit
 from qiskit.circuit import Qubit
@@ -32,7 +33,6 @@ from qiskit.quantum_info.random import random_unitary
 from qiskit.quantum_info import Operator
 from qiskit.circuit.library import U1Gate, U2Gate, U3Gate, QFT, DCXGate, PauliGate
 from qiskit.circuit.gate import Gate
-from qiskit.version import VERSION as current_version_str
 
 try:
     from qiskit.qpy import dump, load
@@ -80,6 +80,12 @@ VERSION_PATTERN = (
 )
 
 
+def version_release_parts(version: str):
+    """The "release" component of a valid Python version string, as a tuple of integers."""
+    version_match = re.search(VERSION_PATTERN, version, re.VERBOSE | re.IGNORECASE)
+    return tuple(int(x) for x in version_match.group("release").split("."))
+
+
 def generate_full_circuit():
     """Generate a multiregister circuit with name, metadata, phase."""
     qr_a = QuantumRegister(4, "a")
@@ -324,7 +330,7 @@ def generate_parameter_vector_expression():  # pylint: disable=invalid-name
     qc = QuantumCircuit(7, name="vector_expansion")
     entanglement = [[i, i + 1] for i in range(7 - 1)]
     input_params = ParameterVector("x_par", 14)
-    user_params = ParameterVector("\u03B8_par", 1)
+    user_params = ParameterVector("\u03b8_par", 1)
 
     for i in range(qc.num_qubits):
         qc.ry(user_params[0], qc.qubits[i])
@@ -554,11 +560,14 @@ def generate_calibrated_circuits():
     return circuits
 
 
-def generate_controlled_gates():
+def generate_controlled_gates(version):
     """Test QPY serialization with custom ControlledGates."""
     circuits = []
     qc = QuantumCircuit(3, name="custom_controlled_gates")
-    controlled_gate = DCXGate().control(1)
+    if version >= (2, 3, 0):
+        controlled_gate = DCXGate().control(1, annotated=False)
+    else:
+        controlled_gate = DCXGate().control(1)
     qc.append(controlled_gate, [0, 1, 2])
     circuits.append(qc)
     custom_gate = Gate("black_box", 1, [])
@@ -568,7 +577,10 @@ def generate_controlled_gates():
     custom_gate.definition = custom_definition
     nested_qc = QuantumCircuit(3, name="nested_qc")
     qc.append(custom_gate, [0])
-    controlled_gate = custom_gate.control(2)
+    if version >= (2, 3, 0):
+        controlled_gate = custom_gate.control(2, annotated=False)
+    else:
+        controlled_gate = custom_gate.control(2)
     nested_qc.append(controlled_gate, [0, 1, 2])
     circuits.append(nested_qc)
     qc_open = QuantumCircuit(2, name="open_cx")
@@ -577,11 +589,14 @@ def generate_controlled_gates():
     return circuits
 
 
-def generate_open_controlled_gates():
+def generate_open_controlled_gates(version):
     """Test QPY serialization with custom ControlledGates with open controls."""
     circuits = []
     qc = QuantumCircuit(3, name="open_controls_simple")
-    controlled_gate = DCXGate().control(1, ctrl_state=0)
+    if version >= (2, 3, 0):
+        controlled_gate = DCXGate().control(1, ctrl_state=0, annotated=False)
+    else:
+        controlled_gate = DCXGate().control(1, ctrl_state=0)
     qc.append(controlled_gate, [0, 1, 2])
     circuits.append(qc)
 
@@ -592,7 +607,10 @@ def generate_open_controlled_gates():
     custom_gate.definition = custom_definition
     nested_qc = QuantumCircuit(3, name="open_controls_nested")
     nested_qc.append(custom_gate, [0])
-    controlled_gate = custom_gate.control(2, ctrl_state=1)
+    if version >= (2, 3, 0):
+        controlled_gate = custom_gate.control(2, ctrl_state=1, annotated=False)
+    else:
+        controlled_gate = custom_gate.control(2, ctrl_state=1)
     nested_qc.append(controlled_gate, [0, 1, 2])
     circuits.append(nested_qc)
 
@@ -967,8 +985,10 @@ def generate_circuits(version_parts, current_version, load_context=False):
     if version_parts >= (0, 24, 0):
         output_circuits["control_flow_switch.qpy"] = generate_control_flow_switch_circuits()
     if version_parts >= (0, 24, 1):
-        output_circuits["open_controlled_gates.qpy"] = generate_open_controlled_gates()
-        output_circuits["controlled_gates.qpy"] = generate_controlled_gates()
+        output_circuits["open_controlled_gates.qpy"] = generate_open_controlled_gates(
+            current_version
+        )
+        output_circuits["controlled_gates.qpy"] = generate_controlled_gates(current_version)
     if version_parts >= (0, 24, 2):
         output_circuits["layout.qpy"] = generate_layout_circuits()
     if version_parts >= (0, 25, 0) and version_parts < (2, 0):
@@ -1151,18 +1171,12 @@ def _main():
     )
     args = parser.parse_args()
 
-    current_version = current_version_str.split(".")
-    for i in range(len(current_version[2])):
-        if current_version[2][i].isalpha():
-            current_version[2] = current_version[2][:i]
-            break
-    current_version = tuple(int(x) for x in current_version)
+    current_version = version_release_parts(qiskit.__version__)
 
     # Terra 0.18.0 was the first release with QPY, so that's the default.
     version_parts = (0, 18, 0)
     if args.version:
-        version_match = re.search(VERSION_PATTERN, args.version, re.VERBOSE | re.IGNORECASE)
-        version_parts = tuple(int(x) for x in version_match.group("release").split("."))
+        version_parts = version_release_parts(args.version)
 
     if args.command == "generate":
         qpy_files = generate_circuits(version_parts, current_version)
